@@ -31,6 +31,7 @@
 #include "json_stats.h"
 #include "svg_stats.h"
 #include "raw_stats.h"
+#include "pcp_stats.h"
 #endif
 
 /*
@@ -67,7 +68,8 @@ struct act_bitmap irq_bitmap = {
 struct activity cpu_act = {
 	.id		= A_CPU,
 	.options	= AO_COLLECTED + AO_COUNTED + AO_PERSISTENT +
-			  AO_MULTIPLE_OUTPUTS + AO_GRAPH_PER_ITEM,
+			  AO_MULTIPLE_OUTPUTS + AO_GRAPH_PER_ITEM +
+			  AO_ALWAYS_COUNTED,
 	.magic		= ACTIVITY_MAGIC_BASE + 1,
 	.group		= G_DEFAULT,
 #ifdef SOURCE_SADC
@@ -91,6 +93,7 @@ struct activity cpu_act = {
 	.f_json_print	= json_print_cpu_stats,
 	.f_svg_print	= svg_print_cpu_stats,
 	.f_raw_print	= raw_print_cpu_stats,
+	.f_pcp_print	= pcp_print_cpu_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "CPU utilization",
@@ -136,6 +139,7 @@ struct activity pcsw_act = {
 	.f_json_print	= json_print_pcsw_stats,
 	.f_svg_print	= svg_print_pcsw_stats,
 	.f_raw_print	= raw_print_pcsw_stats,
+	.f_pcp_print	= pcp_print_pcsw_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "Task creation and switching activity",
@@ -181,6 +185,7 @@ struct activity irq_act = {
 	.f_json_print	= json_print_irq_stats,
 	.f_svg_print	= NULL,
 	.f_raw_print	= raw_print_irq_stats,
+	.f_pcp_print	= pcp_print_irq_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "Interrupts statistics",
@@ -226,6 +231,7 @@ struct activity swap_act = {
 	.f_json_print	= json_print_swap_stats,
 	.f_svg_print	= svg_print_swap_stats,
 	.f_raw_print	= raw_print_swap_stats,
+	.f_pcp_print	= pcp_print_swap_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "Swap activity",
@@ -272,6 +278,7 @@ struct activity paging_act = {
 	.f_json_print	= json_print_paging_stats,
 	.f_svg_print	= svg_print_paging_stats,
 	.f_raw_print	= raw_print_paging_stats,
+	.f_pcp_print	= pcp_print_paging_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "Paging activity",
@@ -317,6 +324,7 @@ struct activity io_act = {
 	.f_json_print	= json_print_io_stats,
 	.f_svg_print	= svg_print_io_stats,
 	.f_raw_print	= raw_print_io_stats,
+	.f_pcp_print	= pcp_print_io_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "I/O and transfer rate statistics",
@@ -363,6 +371,7 @@ struct activity memory_act = {
 	.f_json_print	= json_print_memory_stats,
 	.f_svg_print	= svg_print_memory_stats,
 	.f_raw_print	= raw_print_memory_stats,
+	.f_pcp_print	= pcp_print_memory_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "Memory and/or swap utilization",
@@ -408,6 +417,7 @@ struct activity ktables_act = {
 	.f_json_print	= json_print_ktables_stats,
 	.f_svg_print	= svg_print_ktables_stats,
 	.f_raw_print	= raw_print_ktables_stats,
+	.f_pcp_print	= pcp_print_ktables_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "Kernel tables statistics",
@@ -453,6 +463,7 @@ struct activity queue_act = {
 	.f_json_print	= json_print_queue_stats,
 	.f_svg_print	= svg_print_queue_stats,
 	.f_raw_print	= raw_print_queue_stats,
+	.f_pcp_print	= pcp_print_queue_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "Queue length and load average statistics",
@@ -498,6 +509,7 @@ struct activity serial_act = {
 	.f_json_print	= json_print_serial_stats,
 	.f_svg_print	= NULL,
 	.f_raw_print	= raw_print_serial_stats,
+	.f_pcp_print	= pcp_print_serial_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "TTY devices statistics",
@@ -588,6 +600,7 @@ struct activity net_dev_act = {
 	.f_json_print	= json_print_net_dev_stats,
 	.f_svg_print	= svg_print_net_dev_stats,
 	.f_raw_print	= raw_print_net_dev_stats,
+	.f_pcp_print	= pcp_print_net_dev_stats,
 	.f_count_new	= count_new_net_dev,
 	.item_list	= NULL,
 	.desc		= "Network interfaces statistics",
@@ -634,6 +647,7 @@ struct activity net_edev_act = {
 	.f_json_print	= json_print_net_edev_stats,
 	.f_svg_print	= svg_print_net_edev_stats,
 	.f_raw_print	= raw_print_net_edev_stats,
+	.f_pcp_print	= pcp_print_net_edev_stats,
 	.f_count_new	= count_new_net_edev,
 	.item_list	= NULL,
 	.desc		= "Network interfaces errors statistics",
@@ -679,6 +693,7 @@ struct activity net_nfs_act = {
 	.f_json_print	= json_print_net_nfs_stats,
 	.f_svg_print	= svg_print_net_nfs_stats,
 	.f_raw_print	= raw_print_net_nfs_stats,
+	.f_pcp_print	= pcp_print_net_nfs_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "NFS client statistics",
@@ -725,6 +740,7 @@ struct activity net_nfsd_act = {
 	.f_json_print	= json_print_net_nfsd_stats,
 	.f_svg_print	= svg_print_net_nfsd_stats,
 	.f_raw_print	= raw_print_net_nfsd_stats,
+	.f_pcp_print	= pcp_print_net_nfsd_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "NFS server statistics",
@@ -770,6 +786,7 @@ struct activity net_sock_act = {
 	.f_json_print	= json_print_net_sock_stats,
 	.f_svg_print	= svg_print_net_sock_stats,
 	.f_raw_print	= raw_print_net_sock_stats,
+	.f_pcp_print	= pcp_print_net_sock_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "IPv4 sockets statistics",
@@ -815,6 +832,7 @@ struct activity net_ip_act = {
 	.f_json_print	= json_print_net_ip_stats,
 	.f_svg_print	= svg_print_net_ip_stats,
 	.f_raw_print	= raw_print_net_ip_stats,
+	.f_pcp_print	= pcp_print_net_ip_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "IPv4 traffic statistics",
@@ -860,6 +878,7 @@ struct activity net_eip_act = {
 	.f_json_print	= json_print_net_eip_stats,
 	.f_svg_print	= svg_print_net_eip_stats,
 	.f_raw_print	= raw_print_net_eip_stats,
+	.f_pcp_print	= pcp_print_net_eip_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "IPv4 traffic errors statistics",
@@ -906,6 +925,7 @@ struct activity net_icmp_act = {
 	.f_json_print	= json_print_net_icmp_stats,
 	.f_svg_print	= svg_print_net_icmp_stats,
 	.f_raw_print	= raw_print_net_icmp_stats,
+	.f_pcp_print	= pcp_print_net_icmp_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "ICMPv4 traffic statistics",
@@ -952,6 +972,7 @@ struct activity net_eicmp_act = {
 	.f_json_print	= json_print_net_eicmp_stats,
 	.f_svg_print	= svg_print_net_eicmp_stats,
 	.f_raw_print	= raw_print_net_eicmp_stats,
+	.f_pcp_print	= pcp_print_net_eicmp_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "ICMPv4 traffic errors statistics",
@@ -997,6 +1018,7 @@ struct activity net_tcp_act = {
 	.f_json_print	= json_print_net_tcp_stats,
 	.f_svg_print	= svg_print_net_tcp_stats,
 	.f_raw_print	= raw_print_net_tcp_stats,
+	.f_pcp_print	= pcp_print_net_tcp_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "TCPv4 traffic statistics",
@@ -1042,6 +1064,7 @@ struct activity net_etcp_act = {
 	.f_json_print	= json_print_net_etcp_stats,
 	.f_svg_print	= svg_print_net_etcp_stats,
 	.f_raw_print	= raw_print_net_etcp_stats,
+	.f_pcp_print	= pcp_print_net_etcp_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "TCPv4 traffic errors statistics",
@@ -1087,6 +1110,7 @@ struct activity net_udp_act = {
 	.f_json_print	= json_print_net_udp_stats,
 	.f_svg_print	= svg_print_net_udp_stats,
 	.f_raw_print	= raw_print_net_udp_stats,
+	.f_pcp_print	= pcp_print_net_udp_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "UDPv4 traffic statistics",
@@ -1132,6 +1156,7 @@ struct activity net_sock6_act = {
 	.f_json_print	= json_print_net_sock6_stats,
 	.f_svg_print	= svg_print_net_sock6_stats,
 	.f_raw_print	= raw_print_net_sock6_stats,
+	.f_pcp_print	= pcp_print_net_sock6_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "IPv6 sockets statistics",
@@ -1178,6 +1203,7 @@ struct activity net_ip6_act = {
 	.f_json_print	= json_print_net_ip6_stats,
 	.f_svg_print	= svg_print_net_ip6_stats,
 	.f_raw_print	= raw_print_net_ip6_stats,
+	.f_pcp_print	= pcp_print_net_ip6_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "IPv6 traffic statistics",
@@ -1224,6 +1250,7 @@ struct activity net_eip6_act = {
 	.f_json_print	= json_print_net_eip6_stats,
 	.f_svg_print	= svg_print_net_eip6_stats,
 	.f_raw_print	= raw_print_net_eip6_stats,
+	.f_pcp_print	= pcp_print_net_eip6_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "IPv6 traffic errors statistics",
@@ -1271,6 +1298,7 @@ struct activity net_icmp6_act = {
 	.f_json_print	= json_print_net_icmp6_stats,
 	.f_svg_print	= svg_print_net_icmp6_stats,
 	.f_raw_print	= raw_print_net_icmp6_stats,
+	.f_pcp_print	= pcp_print_net_icmp6_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "ICMPv6 traffic statistics",
@@ -1317,6 +1345,7 @@ struct activity net_eicmp6_act = {
 	.f_json_print	= json_print_net_eicmp6_stats,
 	.f_svg_print	= svg_print_net_eicmp6_stats,
 	.f_raw_print	= raw_print_net_eicmp6_stats,
+	.f_pcp_print	= pcp_print_net_eicmp6_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "ICMPv6 traffic errors statistics",
@@ -1362,6 +1391,7 @@ struct activity net_udp6_act = {
 	.f_json_print	= json_print_net_udp6_stats,
 	.f_svg_print	= svg_print_net_udp6_stats,
 	.f_raw_print	= raw_print_net_udp6_stats,
+	.f_pcp_print	= pcp_print_net_udp6_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "UDPv6 traffic statistics",
@@ -1407,6 +1437,7 @@ struct activity pwr_cpufreq_act = {
 	.f_json_print	= json_print_pwr_cpufreq_stats,
 	.f_svg_print	= svg_print_pwr_cpufreq_stats,
 	.f_raw_print	= raw_print_pwr_cpufreq_stats,
+	.f_pcp_print	= pcp_print_pwr_cpufreq_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "CPU clock frequency",
@@ -1814,6 +1845,7 @@ struct activity softnet_act = {
 	.f_json_print	= json_print_softnet_stats,
 	.f_svg_print	= svg_print_softnet_stats,
 	.f_raw_print	= raw_print_softnet_stats,
+	.f_pcp_print	= pcp_print_softnet_stats,
 	.f_count_new	= NULL,
 	.item_list	= NULL,
 	.desc		= "Software-based network processing statistics",
