@@ -228,6 +228,7 @@
 #define K_SHOWTOC	"showtoc"
 #define K_CUSTOMCOL	"customcol"
 #define K_BWCOL		"bwcol"
+#define K_PCPARCHIVE	"pcparchive="
 
 /* Environment variables */
 #define ENV_COLORS_PALETTE	"S_COLORS_PALETTE"
@@ -704,6 +705,11 @@ struct record_header {
  * line for this activity (see options --dev=, --iface=, ...)
  */
 #define AO_LIST_ON_CMDLINE	0x100
+/*
+ * Indicate that the number of items for this activity should always
+ * be counted, even if the activity is not collected.
+ */
+#define AO_ALWAYS_COUNTED	0x200
 
 #define IS_COLLECTED(m)		(((m) & AO_COLLECTED)        == AO_COLLECTED)
 #define IS_SELECTED(m)		(((m) & AO_SELECTED)         == AO_SELECTED)
@@ -714,6 +720,7 @@ struct record_header {
 #define ONE_GRAPH_PER_ITEM(m)	(((m) & AO_GRAPH_PER_ITEM)   == AO_GRAPH_PER_ITEM)
 #define IS_MATRIX(m)		(((m) & AO_MATRIX)           == AO_MATRIX)
 #define HAS_LIST_ON_CMDLINE(m)	(((m) & AO_LIST_ON_CMDLINE)  == AO_LIST_ON_CMDLINE)
+#define ALWAYS_COUNT_ITEMS(m)	(((m) & AO_ALWAYS_COUNTED)   == AO_ALWAYS_COUNTED)
 
 #define _buf0	buf[0]
 #define _nr0	nr[0]
@@ -816,6 +823,11 @@ struct activity {
 	 * This function is used by sadf to display activity statistics in raw format.
 	 */
 	__print_funct_t (*f_raw_print) (struct activity *, char *, int);
+	/*
+	 * This function is used by sadf to display activity statistics in PCP format.
+	 */
+	__print_funct_t (*f_pcp_print) (struct activity *, int, unsigned long long,
+					struct record_header *);
 	/*
 	 * This function is used by sadf to count the number of new items in current
 	 * sample and add them to the linked list @item_list.
@@ -1009,25 +1021,33 @@ struct report_format {
 				      struct activity * [], unsigned int [], struct file_activity *);
 	/*
 	 * This function defines the statistics part of the report.
-	 * Used only with textual (XML-like) reports.
+	 * Used only with textual (XML-like) reports and PCP archives.
 	 */
-	__printf_funct_t (*f_statistics) (int *, int);
+	__printf_funct_t (*f_statistics) (int *, int, struct activity * [], unsigned int []);
 	/*
 	 * This function defines the timestamp part of the report.
 	 * Used only with textual (XML-like) reports.
 	 */
 	__tm_funct_t (*f_timestamp) (void *, int, char *, char *, unsigned long long,
-				     struct file_header *, unsigned int);
+				     struct record_header *, struct file_header *, unsigned int);
 	/*
 	 * This function displays the restart messages.
 	 */
-	__printf_funct_t (*f_restart) (int *, int, char *, char *, int, struct file_header *);
+	__printf_funct_t (*f_restart) (int *, int, char *, char *, int,
+				       struct file_header *, struct record_header *);
 	/*
 	 * This function displays the comments.
 	 */
-	__printf_funct_t (*f_comment) (int *, int, char *, char *, int, char *, struct file_header *);
+	__printf_funct_t (*f_comment) (int *, int, char *, char *, int, char *,
+				       struct file_header *, struct record_header *);
+	/*
+	 * This is the main function used to display all the statistics for current format.
+	 */
+	void (*f_display) (int, char *, struct file_activity *, struct file_magic *,
+			   struct tm *, struct tm *, void *);
 };
 
+
 /* Possible actions for functions used to display reports */
 #define F_BEGIN	0x01
 #define F_MAIN	0x02
@@ -1320,8 +1340,8 @@ void allocate_structures
 int check_disk_reg
 	(struct activity *, int, int, int);
 void check_file_actlst
-	(int *, char *, struct activity * [], struct file_magic *, struct file_header *,
-	 struct file_activity **, unsigned int [], int, int *, int *);
+	(int *, char *, struct activity * [], unsigned int, struct file_magic *,
+	 struct file_header *, struct file_activity **, unsigned int [], int, int *, int *);
 int check_net_dev_reg
 	(struct activity *, int, int, int);
 int check_net_edev_reg
@@ -1371,9 +1391,10 @@ int parse_timestamp
 void print_report_hdr
 	(unsigned int, struct tm *, struct file_header *);
 void print_sar_comment
-	(int *, int, char *, char *, int, char *, struct file_header *);
+	(int *, int, char *, char *, int, char *, struct file_header *,
+	 struct record_header *);
 __printf_funct_t print_sar_restart
-	(int *, int, char *, char *, int, struct file_header *);
+	(int *, int, char *, char *, int, struct file_header *, struct record_header *);
 int print_special_record
 	(struct record_header *, unsigned int, struct tstamp *, struct tstamp *,
 	 int, int, struct tm *, struct tm *, char *, int, struct file_magic *,
