@@ -38,6 +38,11 @@
 # define _(string) (string)
 #endif
 
+#ifdef HAVE_PCP
+#include <pcp/pmapi.h>
+#include <pcp/import.h>
+#endif
+
 #ifdef USE_SCCSID
 #define SCCSID "@(#)sysstat-" VERSION ": " __FILE__ " compiled " __DATE__ " " __TIME__
 char *sccsid(void) { return (SCCSID); }
@@ -94,7 +99,7 @@ void usage(char *progname)
 		progname);
 
 	fprintf(stderr, _("Options are:\n"
-			  "[ -C ] [ -c | -d | -g | -j | -p | -r | -x ] [ -H ] [ -h ] [ -T | -t | -U ] [ -V ]\n"
+			  "[ -C ] [ -c | -d | -g | -j | -l | -p | -r | -x ] [ -H ] [ -h ] [ -T | -t | -U ] [ -V ]\n"
 			  "[ -O <opts> [,...] ] [ -P { <cpu> [,...] | ALL } ]\n"
 			  "[ --dev=<dev_list> ] [ --fs=<fs_list> ] [ --iface=<iface_list> ]\n"
 			  "[ -s [ <hh:mm[:ss]> ] ] [ -e [ <hh:mm[:ss]> ] ]\n"
@@ -121,19 +126,18 @@ void init_structures(void)
  * Look for output format in array.
  *
  * IN:
- * @fmt		Array of output formats.
- * @format	Output format to look for.
+ * @oformat	Output format to look for.
  *
  * RETURNS:
  * Position of output format in array.
  ***************************************************************************
  */
-int get_format_position(struct report_format *fmt[], unsigned int format)
+int get_format_position(unsigned int oformat)
 {
         int i;
 
         for (i = 0; i < NR_FMT; i++) {
-                if (fmt[i]->id == format)
+                if (fmt[i]->id == oformat)
                         break;
         }
 
@@ -164,7 +168,7 @@ void check_format_options(void)
 	}
 
 	/* Get format position in array */
-	f_position = get_format_position(fmt, format);
+	f_position = get_format_position(format);
 
 	/* Check options consistency wrt output format */
 	if (!ACCEPT_HEADER_ONLY(fmt[f_position]->options)) {
@@ -711,8 +715,8 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 				    cur_date, cur_time, TIMESTAMP_LEN, rectime);
 
 	if (*fmt[f_position]->f_timestamp) {
-		pre = (char *) (*fmt[f_position]->f_timestamp)(parm, F_BEGIN, cur_date, cur_time,
-							       dt, &file_hdr, flags);
+		pre = (char *) (*fmt[f_position]->f_timestamp)(parm, F_BEGIN, cur_date, cur_time, dt,
+							       &record_hdr[curr], &file_hdr, flags);
 	}
 
 	/* Display statistics */
@@ -732,7 +736,8 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 
 					if (*fmt[f_position]->f_timestamp) {
 						(*fmt[f_position]->f_timestamp)(tab, F_MAIN, cur_date, cur_time,
-										dt, &file_hdr, flags);
+										dt, &record_hdr[curr],
+										&file_hdr, flags);
 					}
 				}
 				(*act[i]->f_json_print)(act[i], curr, *tab, itv);
@@ -763,6 +768,13 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 				(*act[i]->f_raw_print)(act[i], pre, curr);
 			}
 
+			else if (format == F_PCP_OUTPUT) {
+				/* PCP archive */
+				if (*act[i]->f_pcp_print) {
+					(*act[i]->f_pcp_print)(act[i], curr, itv, &record_hdr[curr]);
+				}
+			}
+
 			else {
 				/* Other output formats: db, ppc */
 				(*act[i]->f_render)(act[i], (format == F_DB_OUTPUT), pre, curr, itv);
@@ -771,8 +783,8 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 	}
 
 	if (*fmt[f_position]->f_timestamp) {
-		(*fmt[f_position]->f_timestamp)(parm, F_END, cur_date, cur_time,
-						dt, &file_hdr, flags);
+		(*fmt[f_position]->f_timestamp)(parm, F_END, cur_date, cur_time, dt,
+						&record_hdr[curr], &file_hdr, flags);
 	}
 
 	return 1;
@@ -981,77 +993,104 @@ void display_curr_act_graphs(int ifd, int *curr, long *cnt, int *eosaf,
  ***************************************************************************
  * Display file contents in selected format (logic #1).
  * Logic #1:	Grouped by record type. Sorted by timestamp.
- * Formats:	XML, JSON
+ * Formats:	XML, JSON, PCP
+ *
+ * NB: all statistics data will be sorted by timestamp.
+ * Example: If stats for activities A and B at time t and t' have been collected,
+ * the output will be:
+ * stats for activity A at t
+ * stats for activity B at t
+ * stats for activity A at t'
+ * stats for activity B at t'
  *
  * IN:
  * @ifd		File descriptor of input file.
- * @file_actlst	List of (known or unknown) activities in file.
  * @file	System activity data file name (name of file being read).
+ * @file_actlst	List of (known or unknown) activities in file.
  * @file_magic	System activity file magic header.
  * @rectime	Structure where timestamp (expressed in local time or in UTC
  *		depending on whether options -T/-t have been used or not) can
  *		be saved for current record.
  * @loctime	Structure where timestamp (expressed in local time) can be
  *		saved for current record.
+ * @dparm	PCP archive file name.
  ***************************************************************************
  */
-void logic1_display_loop(int ifd, struct file_activity *file_actlst, char *file,
-			 struct file_magic *file_magic,
-			 struct tm *rectime, struct tm *loctime)
+void logic1_display_loop(int ifd, char *file, struct file_activity *file_actlst,
+			 struct file_magic *file_magic, struct tm *rectime,
+			 struct tm *loctime, void *dparm)
 {
-	int curr, tab = 0, rtype;
+	int curr, rtype, tab = 0;
 	int eosaf, next, reset = FALSE;
+	int ign_flag = IGNORE_COMMENT + IGNORE_RESTART;
 	long cnt = 1;
+	char *pcparchive = (char *) dparm;
 
-	if (format == F_JSON_OUTPUT) {
-		/* Use a decimal point to make JSON code compliant with RFC7159 */
-		setlocale(LC_NUMERIC, "C");
+	if (CREATE_ITEM_LIST(fmt[f_position]->options)) {
+		/* Count items in file (e.g. for PCP output) */
+		if (!count_file_items(ifd, file, file_magic, file_actlst, rectime, loctime))
+			/* No record to display */
+			return;
 	}
-
 	/* Save current file position */
 	seek_file_position(ifd, DO_SAVE);
 
 	/* Print header (eg. XML file header) */
 	if (*fmt[f_position]->f_header) {
-		(*fmt[f_position]->f_header)(&tab, F_BEGIN, file, file_magic,
+		(*fmt[f_position]->f_header)(&tab, F_BEGIN, pcparchive, file_magic,
 					     &file_hdr, act, id_seq, file_actlst);
 	}
 
+	if (ORDER_ALL_RECORDS(fmt[f_position]->options)) {
+		ign_flag = IGNORE_NOTHING;
+
+		/* RESTART and COMMENTS records will be immediately processed */
+		if (*fmt[f_position]->f_restart) {
+			(*fmt[f_position]->f_restart)(&tab, F_BEGIN, NULL, NULL, FALSE,
+						      &file_hdr, NULL);
+		}
+		if (DISPLAY_COMMENT(flags) && (*fmt[f_position]->f_comment)) {
+			(*fmt[f_position]->f_comment)(&tab, F_BEGIN, NULL, NULL, 0, NULL,
+						      &file_hdr, NULL);
+		}
+	}
+
 	/* Process activities */
 	if (*fmt[f_position]->f_statistics) {
-		(*fmt[f_position]->f_statistics)(&tab, F_BEGIN);
+		(*fmt[f_position]->f_statistics)(&tab, F_BEGIN, act, id_seq);
 	}
 
 	do {
 		/*
 		 * If this record is a special (RESTART or COMMENT) one,
-		 * skip it and try to read the next record in file.
+		 * process it then try to read the next record in file.
 		 */
 		do {
-			eosaf = read_next_sample(ifd, IGNORE_COMMENT | IGNORE_RESTART, 0,
-						 file, &rtype, tab, file_magic, file_actlst,
+			eosaf = read_next_sample(ifd, ign_flag, 0, file,
+						 &rtype, tab, file_magic, file_actlst,
 						 rectime, loctime, UEOF_STOP);
 		}
 		while (!eosaf && ((rtype == R_RESTART) || (rtype == R_COMMENT) ||
 			(tm_start.use && (datecmp(loctime, &tm_start) < 0)) ||
 			(tm_end.use && (datecmp(loctime, &tm_end) >= 0))));
 
-		/* Save the first stats collected. Used for example in next_slice() function */
-		copy_structures(act, id_seq, record_hdr, 2, 0);
-
 		curr = 1;
 		cnt = count;
 		reset = TRUE;
 
 		if (!eosaf) {
+
+			/* Save the first stats collected. Used for example in next_slice() function */
+			copy_structures(act, id_seq, record_hdr, 2, 0);
+
 			do {
-				eosaf = read_next_sample(ifd, IGNORE_COMMENT | IGNORE_RESTART, curr,
-							 file, &rtype, tab, file_magic, file_actlst,
+				eosaf = read_next_sample(ifd, ign_flag, curr, file,
+							 &rtype, tab, file_magic, file_actlst,
 							 rectime, loctime, UEOF_CONT);
 
 				if (!eosaf && (rtype != R_COMMENT) && (rtype != R_RESTART)) {
 					if (*fmt[f_position]->f_statistics) {
-						(*fmt[f_position]->f_statistics)(&tab, F_MAIN);
+						(*fmt[f_position]->f_statistics)(&tab, F_MAIN, act, id_seq);
 					}
 
 					/* next is set to 1 when we were close enough to desired interval */
@@ -1073,8 +1112,8 @@ void logic1_display_loop(int ifd, struct file_activity *file_actlst, char *file,
 			if (!cnt) {
 				/* Go to next Linux restart, if possible */
 				do {
-					eosaf = read_next_sample(ifd, IGNORE_COMMENT | IGNORE_RESTART, curr,
-								 file, &rtype, tab, file_magic, file_actlst,
+					eosaf = read_next_sample(ifd, ign_flag, curr, file,
+								 &rtype, tab, file_magic, file_actlst,
 								 rectime, loctime, UEOF_CONT);
 				}
 				while (!eosaf && (rtype != R_RESTART));
@@ -1085,7 +1124,23 @@ void logic1_display_loop(int ifd, struct file_activity *file_actlst, char *file,
 	while (!eosaf);
 
 	if (*fmt[f_position]->f_statistics) {
-		(*fmt[f_position]->f_statistics)(&tab, F_END);
+		(*fmt[f_position]->f_statistics)(&tab, F_END, act, id_seq);
+	}
+
+	if (ign_flag == IGNORE_NOTHING) {
+		/*
+		 * RESTART and COMMENT records have already been processed.
+		 * Display possible trailing data then terminate.
+		 */
+		if (*fmt[f_position]->f_restart) {
+			(*fmt[f_position]->f_restart)(&tab, F_END, NULL, NULL,
+						      FALSE, &file_hdr, NULL);
+		}
+		if (DISPLAY_COMMENT(flags) && (*fmt[f_position]->f_comment)) {
+			(*fmt[f_position]->f_comment)(&tab, F_END, NULL, NULL, 0, NULL,
+						      &file_hdr, NULL);
+		}
+		goto terminate;
 	}
 
 	/* Rewind file */
@@ -1094,7 +1149,7 @@ void logic1_display_loop(int ifd, struct file_activity *file_actlst, char *file,
 	/* Process now RESTART entries to display restart messages */
 	if (*fmt[f_position]->f_restart) {
 		(*fmt[f_position]->f_restart)(&tab, F_BEGIN, NULL, NULL, FALSE,
-					      &file_hdr);
+					      &file_hdr, NULL);
 	}
 
 	do {
@@ -1105,7 +1160,7 @@ void logic1_display_loop(int ifd, struct file_activity *file_actlst, char *file,
 	while (!eosaf);
 
 	if (*fmt[f_position]->f_restart) {
-		(*fmt[f_position]->f_restart)(&tab, F_END, NULL, NULL, FALSE, &file_hdr);
+		(*fmt[f_position]->f_restart)(&tab, F_END, NULL, NULL, FALSE, &file_hdr, NULL);
 	}
 
 	/* Rewind file */
@@ -1115,7 +1170,7 @@ void logic1_display_loop(int ifd, struct file_activity *file_actlst, char *file,
 	if (DISPLAY_COMMENT(flags)) {
 		if (*fmt[f_position]->f_comment) {
 			(*fmt[f_position]->f_comment)(&tab, F_BEGIN, NULL, NULL, 0, NULL,
-						      &file_hdr);
+						      &file_hdr, NULL);
 		}
 		do {
 			eosaf = read_next_sample(ifd, IGNORE_RESTART, 0,
@@ -1126,13 +1181,14 @@ void logic1_display_loop(int ifd, struct file_activity *file_actlst, char *file,
 
 		if (*fmt[f_position]->f_comment) {
 			(*fmt[f_position]->f_comment)(&tab, F_END, NULL, NULL, 0, NULL,
-						      &file_hdr);
+						      &file_hdr, NULL);
 		}
 	}
 
+terminate:
 	/* Print header trailer */
 	if (*fmt[f_position]->f_header) {
-		(*fmt[f_position]->f_header)(&tab, F_END, file, file_magic,
+		(*fmt[f_position]->f_header)(&tab, F_END, pcparchive, file_magic,
 					     &file_hdr, act, id_seq, file_actlst);
 	}
 }
@@ -1144,21 +1200,31 @@ void logic1_display_loop(int ifd, struct file_activity *file_actlst, char *file,
  * 		records.
  * Formats:	ppc, CSV, raw
  *
+ * NB: All statistics data for one activity will be displayed before
+ * displaying stats for next activity. This is what sar does in its report.
+ * Example: If stats for activities A and B at time t and t' have been collected,
+ * the output will be:
+ * stats for activity A at t
+ * stats for activity A at t'
+ * stats for activity B at t
+ * stats for activity B at t'
+ *
  * IN:
  * @ifd		File descriptor of input file.
+ * @file	Name of file being read.
  * @file_actlst	List of (known or unknown) activities in file.
+ * @file_magic	file_magic structure filled with file magic header data.
  * @rectime	Structure where timestamp (expressed in local time or in UTC
  *		depending on whether options -T/-t have been used or not) can
  *		be saved for current record.
  * @loctime	Structure where timestamp (expressed in local time) can be
  *		saved for current record.
- * @file	Name of file being read.
- * @file_magic	file_magic structure filled with file magic header data.
+ * @dparm	Unused here.
  ***************************************************************************
  */
-void logic2_display_loop(int ifd, struct file_activity *file_actlst,
-			 struct tm *rectime, struct tm *loctime, char *file,
-			 struct file_magic *file_magic)
+void logic2_display_loop(int ifd, char *file, struct file_activity *file_actlst,
+			 struct file_magic *file_magic, struct tm *rectime,
+			 struct tm *loctime, void *dparm)
 {
 	int i, p;
 	int curr = 1, rtype;
@@ -1265,25 +1331,24 @@ void logic2_display_loop(int ifd, struct file_activity *file_actlst,
 
 /*
  ***************************************************************************
- * Display file contents in selected format (logic #3).
- * Logic #3:	Special logic for SVG output format.
- * Formats:	SVG
+ * Display file contents in SVG format.
  *
  * IN:
  * @ifd		File descriptor of input file.
+ * @file	Name of file being read.
  * @file_actlst	List of (known or unknown) activities in file.
+ * @file_magic	file_magic structure filled with file magic header data.
  * @rectime	Structure where timestamp (expressed in local time or in UTC
  *		depending on whether options -T/-t have been used or not) can
  *		be saved for current record.
  * @loctime	Structure where timestamp (expressed in local time) can be
  *		saved for current record.
- * @file	Name of file being read.
- * @file_magic	file_magic structure filled with file magic header data.
+ * @dparm	Unused here.
  ***************************************************************************
  */
-void logic3_display_loop(int ifd, struct file_activity *file_actlst,
-			 struct tm *rectime, struct tm *loctime, char *file,
-			 struct file_magic *file_magic)
+void svg_display_loop(int ifd, char *file, struct file_activity *file_actlst,
+		      struct file_magic *file_magic, struct tm *rectime,
+		      struct tm *loctime, void *dparm)
 {
 	struct svg_hdr_parm parm;
 	int i, p;
@@ -1292,9 +1357,6 @@ void logic3_display_loop(int ifd, struct file_activity *file_actlst,
 	long cnt = 1;
 	int graph_nr = 0;
 
-	/* Use a decimal point to make SVG code locale independent */
-	setlocale(LC_NUMERIC, "C");
-
 	/* Init custom colors palette */
 	init_custom_color_palette();
 
@@ -1402,9 +1464,10 @@ close_svg:
  *
  * IN:
  * @dfile	System activity data file name.
+ * @pcparchive	PCP archive file name.
  ***************************************************************************
  */
-void read_stats_from_file(char dfile[])
+void read_stats_from_file(char dfile[], char pcparchive[])
 {
 	struct file_magic file_magic;
 	struct file_activity *file_actlst = NULL;
@@ -1413,11 +1476,14 @@ void read_stats_from_file(char dfile[])
 
 	/* Prepare file for reading and read its headers */
 	ignore = ACCEPT_BAD_FILE_FORMAT(fmt[f_position]->options);
-	check_file_actlst(&ifd, dfile, act, &file_magic, &file_hdr,
+	check_file_actlst(&ifd, dfile, act, flags, &file_magic, &file_hdr,
 			  &file_actlst, id_seq, ignore, &endian_mismatch, &arch_64);
 
 	if (DISPLAY_HDR_ONLY(flags)) {
 		if (*fmt[f_position]->f_header) {
+			if (format == F_PCP_OUTPUT) {
+				dfile = pcparchive;
+			}
 			/* Display only data file header then exit */
 			(*fmt[f_position]->f_header)(&tab, F_BEGIN + F_END, dfile, &file_magic,
 						     &file_hdr, act, id_seq, file_actlst);
@@ -1428,18 +1494,15 @@ void read_stats_from_file(char dfile[])
 	/* Perform required allocations */
 	allocate_structures(act);
 
+	if (SET_LC_NUMERIC_C(fmt[f_position]->options)) {
+		/* Use a decimal point */
+		setlocale(LC_NUMERIC, "C");
+	}
+
 	/* Call function corresponding to selected output format */
-	if (format == F_SVG_OUTPUT) {
-		logic3_display_loop(ifd, file_actlst,
-				    &rectime, &loctime, dfile, &file_magic);
-	}
-	else if (DISPLAY_GROUPED_STATS(fmt[f_position]->options)) {
-		logic2_display_loop(ifd, file_actlst,
-				    &rectime, &loctime, dfile, &file_magic);
-	}
-	else {
-		logic1_display_loop(ifd, file_actlst, dfile,
-				    &file_magic, &rectime, &loctime);
+	if (*fmt[f_position]->f_display) {
+		(*fmt[f_position]->f_display)(ifd, dfile, file_actlst, &file_magic,
+					      &rectime, &loctime, pcparchive);
 	}
 
 	close(ifd);
@@ -1458,13 +1521,13 @@ int main(int argc, char **argv)
 	int opt = 1, sar_options = 0;
 	int day_offset = 0;
 	int i, rc, p, q;
-	char dfile[MAX_FILE_LEN];
+	char dfile[MAX_FILE_LEN], pcparchive[MAX_FILE_LEN];
 	char *t, *v;
 
 	/* Compute page shift in kB */
 	get_kb_shift();
 
-	dfile[0] = '\0';
+	dfile[0] = pcparchive[0] = '\0';
 
 #ifdef USE_NLS
 	/* Init National Language Support */
@@ -1577,6 +1640,11 @@ int main(int argc, char **argv)
 				else if (!strcmp(t, K_BWCOL)) {
 					palette = SVG_BW_COL_PALETTE;
 				}
+				else if (!strncmp(t, K_PCPARCHIVE, strlen(K_PCPARCHIVE))) {
+					v = t + strlen(K_PCPARCHIVE);
+					strncpy(pcparchive, v, MAX_FILE_LEN);
+					pcparchive[MAX_FILE_LEN - 1] = '\0';
+				}
 				else {
 					usage(argv[0]);
 				}
@@ -1676,6 +1744,13 @@ int main(int argc, char **argv)
 						format = F_JSON_OUTPUT;
 						break;
 
+					case 'l':
+						if (format) {
+							usage(argv[0]);
+						}
+						format = F_PCP_OUTPUT;
+						break;
+
 					case 'p':
 						if (format) {
 							usage(argv[0]);
@@ -1769,6 +1844,18 @@ int main(int argc, char **argv)
 		set_default_file(dfile, day_offset, -1);
 	}
 
+	/* PCP mode: If no archive file specified then use the name of the daily data file */
+	if (!pcparchive[0] && (format == F_PCP_OUTPUT)) {
+		strcpy(pcparchive, dfile);
+	}
+
+#ifndef HAVE_PCP
+	if (format == F_PCP_OUTPUT) {
+		fprintf(stderr, _("PCP support not compiled in\n"));
+		exit(1);
+	}
+#endif
+
 	if (tm_start.use && tm_end.use && (tm_end.tm_hour < tm_start.tm_hour)) {
 		tm_end.tm_hour += 24;
 	}
@@ -1807,7 +1894,7 @@ int main(int argc, char **argv)
 	}
 	else {
 		/* Read stats from file */
-		read_stats_from_file(dfile);
+		read_stats_from_file(dfile, pcparchive);
 	}
 
 	/* Free bitmaps */
