@@ -21,8 +21,10 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
 
 #include "sadf.h"
+#include "pcp_def_metrics.h"
 
 #ifdef USE_NLS
 #include <locale.h>
@@ -32,12 +34,66 @@
 #define _(string) (string)
 #endif
 
+#ifdef HAVE_PCP
+#include <pcp/pmapi.h>
+#include <pcp/import.h>
+#endif
+
+extern char *tzname[2];
+
 extern unsigned int flags;
 extern char *seps[];
 
 extern int palette;
 extern unsigned int svg_colors[][SVG_COL_PALETTE_SIZE];
 
+/*
+ ***************************************************************************
+ * Flush data to PCP archive.
+ *
+ * IN:
+ * @record_hdr	Record header for current sample.
+ * @flags	Flags for common options.
+ ***************************************************************************
+ */
+void pcp_write_data(struct record_header *record_hdr, unsigned int flags)
+{
+#ifdef HAVE_PCP
+	int rc;
+	struct tm lrectime;
+	unsigned long long utc_sec = record_hdr->ust_time;
+	static long long delta_utc = LONG_MAX;
+
+	if (!PRINT_LOCAL_TIME(flags)) {
+		if (delta_utc == LONG_MAX) {
+			/* Convert a time_t value from local time to UTC */
+			if (gmtime_r((const time_t *) &(record_hdr->ust_time), &lrectime)) {
+				utc_sec = mktime(&lrectime);
+				delta_utc = utc_sec - record_hdr->ust_time;
+			}
+		}
+		else {
+			/*
+			 * Once pmiWrite() has been called (after the first stats sample),
+			 * subsequent mktime() calls will not give the same result with
+			 * the same input data. So compute a time shift that will be used
+			 * for the next samples.
+			 * We should (really) be careful if pmiWrite() was to be used sooner
+			 * than for the first stats sample (e.g. if we want to save a
+			 * LINUX RESTART record heading the file).
+			 */
+			utc_sec += delta_utc;
+		}
+	}
+
+	/* Write data to PCP archive */
+	if ((rc = pmiWrite(utc_sec, 0)) < 0) {
+		fprintf(stderr, "PCP: pmiWrite: %s\n", pmiErrStr(rc));
+		exit(4);
+	}
+#endif
+}
+
 /*
  ***************************************************************************
  * Display restart messages (database and ppc formats).
@@ -76,10 +132,12 @@ void print_dbppc_restart(char *cur_date, char *cur_time, int utc, char sep,
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC.
  * @file_hdr	System activity file standard header.
+ * @record_hdr	Current record header (unused here).
  ***************************************************************************
  */
 __printf_funct_t print_db_restart(int *tab, int action, char *cur_date,
-				  char *cur_time, int utc, struct file_header *file_hdr)
+				  char *cur_time, int utc, struct file_header *file_hdr,
+				  struct record_header *record_hdr)
 {
 	/* Actions F_BEGIN and F_END ignored */
 	if (action == F_MAIN) {
@@ -98,10 +156,12 @@ __printf_funct_t print_db_restart(int *tab, int action, char *cur_date,
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC.
  * @file_hdr	System activity file standard header.
+ * @record_hdr	Current record header (unused here).
  ***************************************************************************
  */
 __printf_funct_t print_ppc_restart(int *tab, int action, char *cur_date,
-				   char *cur_time, int utc, struct file_header *file_hdr)
+				   char *cur_time, int utc, struct file_header *file_hdr,
+				   struct record_header *record_hdr)
 {
 	/* Actions F_BEGIN and F_END ignored */
 	if (action == F_MAIN) {
@@ -120,13 +180,15 @@ __printf_funct_t print_ppc_restart(int *tab, int action, char *cur_date,
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC.
  * @file_hdr	System activity file standard header.
+ * @record_hdr	Current record header (unused here).
  *
  * OUT:
  * @tab		Number of tabulations.
  ***************************************************************************
  */
 __printf_funct_t print_xml_restart(int *tab, int action, char *cur_date,
-				   char *cur_time, int utc, struct file_header *file_hdr)
+				   char *cur_time, int utc, struct file_header *file_hdr,
+				   struct record_header *record_hdr)
 {
 	if (action & F_BEGIN) {
 		xprintf((*tab)++, "<restarts>");
@@ -152,13 +214,15 @@ __printf_funct_t print_xml_restart(int *tab, int action, char *cur_date,
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC.
  * @file_hdr	System activity file standard header.
+ * @record_hdr	Current record header (unused here).
  *
  * OUT:
  * @tab		Number of tabulations.
  ***************************************************************************
  */
 __printf_funct_t print_json_restart(int *tab, int action, char *cur_date,
-				    char *cur_time, int utc, struct file_header *file_hdr)
+				    char *cur_time, int utc, struct file_header *file_hdr,
+				    struct record_header *record_hdr)
 {
 	static int sep = FALSE;
 
@@ -197,10 +261,12 @@ __printf_funct_t print_json_restart(int *tab, int action, char *cur_date,
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC.
  * @file_hdr	System activity file standard header.
+ * @record_hdr	Current record header (unused here).
  ***************************************************************************
  */
 __printf_funct_t print_raw_restart(int *tab, int action, char *cur_date,
-				   char *cur_time, int utc, struct file_header *file_hdr)
+				   char *cur_time, int utc, struct file_header *file_hdr,
+				   struct record_header *record_hdr)
 {
 	/* Actions F_BEGIN and F_END ignored */
 	if (action == F_MAIN) {
@@ -213,6 +279,54 @@ __printf_funct_t print_raw_restart(int *tab, int action, char *cur_date,
 	}
 }
 
+/*
+ ***************************************************************************
+ * Display restart messages (PCP format).
+ *
+ * IN:
+ * @tab		Number of tabulations (unused here).
+ * @action	Action expected from current function.
+ * @cur_date	Date string of current restart message (unused here).
+ * @cur_time	Time string of current restart message (unused here).
+ * @utc		True if @cur_time is expressed in UTC (unused here).
+ * @file_hdr	System activity file standard header.
+ * @record_hdr	Current record header.
+ ***************************************************************************
+ */
+__printf_funct_t print_pcp_restart(int *tab, int action, char *cur_date, char *cur_time,
+				   int utc, struct file_header *file_hdr,
+				   struct record_header *record_hdr)
+{
+#ifdef HAVE_PCP
+	static int def_metrics = FALSE;
+	char buf[64];
+
+	if (action & F_BEGIN) {
+		if (!def_metrics) {
+			pmiAddMetric("system.restart.count",
+				     PM_IN_NULL, PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
+				     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
+
+			pmiAddMetric("system.restart.ncpu",
+				     PM_IN_NULL, PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
+				     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
+
+			def_metrics = TRUE;
+		}
+	}
+	if (action & F_MAIN) {
+		pmiPutValue("system.restart.count", NULL, "1");
+
+		snprintf(buf, sizeof(buf), "%u",
+			 file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1);
+		pmiPutValue("system.restart.ncpu", NULL, buf);
+
+		/* Write data to PCP archive */
+		pcp_write_data(record_hdr, flags);
+	}
+#endif /* HAVE_PCP */
+}
+
 /*
  ***************************************************************************
  * Display comments (database and ppc formats).
@@ -252,11 +366,12 @@ void print_dbppc_comment(char *cur_date, char *cur_time, int utc,
  * @utc		True if @cur_time is expressed in UTC.
  * @comment	Comment to display.
  * @file_hdr	System activity file standard header.
+ * @record_hdr	Current record header (unused here).
  ***************************************************************************
  */
-__printf_funct_t print_db_comment(int *tab, int action, char *cur_date,
-				  char *cur_time, int utc, char *comment,
-				  struct file_header *file_hdr)
+__printf_funct_t print_db_comment(int *tab, int action, char *cur_date, char *cur_time,
+				  int utc, char *comment, struct file_header *file_hdr,
+				  struct record_header *record_hdr)
 {
 	/* Actions F_BEGIN and F_END ignored */
 	if (action & F_MAIN) {
@@ -277,11 +392,12 @@ __printf_funct_t print_db_comment(int *tab, int action, char *cur_date,
  * @utc		True if @cur_time is expressed in UTC.
  * @comment	Comment to display.
  * @file_hdr	System activity file standard header.
+ * @record_hdr	Current record header (unused here).
  ***************************************************************************
  */
-__printf_funct_t print_ppc_comment(int *tab, int action, char *cur_date,
-				   char *cur_time, int utc, char *comment,
-				   struct file_header *file_hdr)
+__printf_funct_t print_ppc_comment(int *tab, int action, char *cur_date, char *cur_time,
+				   int utc, char *comment, struct file_header *file_hdr,
+				   struct record_header *record_hdr)
 {
 	/* Actions F_BEGIN and F_END ignored */
 	if (action & F_MAIN) {
@@ -302,14 +418,15 @@ __printf_funct_t print_ppc_comment(int *tab, int action, char *cur_date,
  * @utc		True if @cur_time is expressed in UTC.
  * @comment	Comment to display.
  * @file_hdr	System activity file standard header (unused here).
+ * @record_hdr	Current record header (unused here).
  *
  * OUT:
  * @tab		Number of tabulations.
  ***************************************************************************
  */
-__printf_funct_t print_xml_comment(int *tab, int action, char *cur_date,
-				   char *cur_time, int utc, char *comment,
-				   struct file_header *file_hdr)
+__printf_funct_t print_xml_comment(int *tab, int action, char *cur_date, char *cur_time,
+				   int utc, char *comment, struct file_header *file_hdr,
+				   struct record_header *record_hdr)
 {
 	if (action & F_BEGIN) {
 		xprintf((*tab)++, "<comments>");
@@ -335,14 +452,15 @@ __printf_funct_t print_xml_comment(int *tab, int action, char *cur_date,
  * @utc		True if @cur_time is expressed in UTC.
  * @comment	Comment to display.
  * @file_hdr	System activity file standard header (unused here).
+ * @record_hdr	Current record header (unused here).
  *
  * OUT:
  * @tab		Number of tabulations.
  ***************************************************************************
  */
-__printf_funct_t print_json_comment(int *tab, int action, char *cur_date,
-				    char *cur_time, int utc, char *comment,
-				    struct file_header *file_hdr)
+__printf_funct_t print_json_comment(int *tab, int action, char *cur_date, char *cur_time,
+				    int utc, char *comment, struct file_header *file_hdr,
+				    struct record_header *record_hdr)
 {
 	static int sep = FALSE;
 
@@ -383,11 +501,12 @@ __printf_funct_t print_json_comment(int *tab, int action, char *cur_date,
  * @utc		True if @cur_time is expressed in UTC.
  * @comment	Comment to display.
  * @file_hdr	System activity file standard header (unused here).
+ * @record_hdr	Current record header (unused here).
  ***************************************************************************
  */
-__printf_funct_t print_raw_comment(int *tab, int action, char *cur_date,
-				   char *cur_time, int utc, char *comment,
-				   struct file_header *file_hdr)
+__printf_funct_t print_raw_comment(int *tab, int action, char *cur_date, char *cur_time,
+				   int utc, char *comment, struct file_header *file_hdr,
+				   struct record_header *record_hdr)
 {
 	/* Actions F_BEGIN and F_END ignored */
 	if (action & F_MAIN) {
@@ -399,6 +518,46 @@ __printf_funct_t print_raw_comment(int *tab, int action, char *cur_date,
 	}
 }
 
+/*
+ ***************************************************************************
+ * Display comments (PCP format).
+ *
+ * IN:
+ * @tab		Number of tabulations (unused here).
+ * @action	Action expected from current function.
+ * @cur_date	Date string of current restart message (unused here).
+ * @cur_time	Time string of current restart message (unused here).
+ * @utc		True if @cur_time is expressed in UTC (unused here).
+ * @comment	Comment to display.
+ * @file_hdr	System activity file standard header (unused here).
+ * @record_hdr	Current record header.
+ ***************************************************************************
+ */
+__printf_funct_t print_pcp_comment(int *tab, int action, char *cur_date, char *cur_time,
+				   int utc, char *comment, struct file_header *file_hdr,
+				   struct record_header *record_hdr)
+{
+#ifdef HAVE_PCP
+	static int def_metrics = FALSE;
+
+	if (action & F_BEGIN) {
+		if (!def_metrics) {
+			pmiAddMetric("system.comment.value",
+				     PM_IN_NULL, PM_TYPE_STRING, PM_INDOM_NULL, PM_SEM_DISCRETE,
+				     pmiUnits(0, 0, 0, 0, 0, 0));
+
+			def_metrics = TRUE;
+		}
+	}
+	if (action & F_MAIN) {
+		pmiPutValue("system.comment.value", NULL, comment);
+
+		/* Write data to PCP archive */
+		pcp_write_data(record_hdr, flags);
+	}
+#endif /* HAVE_PCP */
+}
+
 /*
  ***************************************************************************
  * Display the "statistics" part of the report (XML format).
@@ -406,12 +565,15 @@ __printf_funct_t print_raw_comment(int *tab, int action, char *cur_date,
  * IN:
  * @tab		Number of tabulations.
  * @action	Action expected from current function.
+ * @act		Array of activities (unused here).
+ * @id_seq	Activity sequence (unused here).
  *
  * OUT:
  * @tab		Number of tabulations.
  ***************************************************************************
  */
-__printf_funct_t print_xml_statistics(int *tab, int action)
+__printf_funct_t print_xml_statistics(int *tab, int action, struct activity *act[],
+				      unsigned int id_seq[])
 {
 	if (action & F_BEGIN) {
 		xprintf((*tab)++, "<statistics>");
@@ -428,12 +590,15 @@ __printf_funct_t print_xml_statistics(int *tab, int action)
  * IN:
  * @tab		Number of tabulations.
  * @action	Action expected from current function.
+ * @act		Array of activities (unused here).
+ * @id_seq	Activity sequence (unused here).
  *
  * OUT:
  * @tab		Number of tabulations.
  ***************************************************************************
  */
-__printf_funct_t print_json_statistics(int *tab, int action)
+__printf_funct_t print_json_statistics(int *tab, int action, struct activity *act[],
+				       unsigned int id_seq[])
 {
 	static int sep = FALSE;
 
@@ -457,6 +622,150 @@ __printf_funct_t print_json_statistics(int *tab, int action)
 	}
 }
 
+/*
+ ***************************************************************************
+ * Display the "statistics" part of the report (PCP format).
+ *
+ * IN:
+ * @tab		Number of tabulations (unused here).
+ * @action	Action expected from current function.
+ * @act		Array of activities.
+ * @id_seq	Activity sequence.
+ ***************************************************************************
+ */
+__printf_funct_t print_pcp_statistics(int *tab, int action, struct activity *act[],
+				      unsigned int id_seq[])
+{
+#ifdef HAVE_PCP
+	int i, p;
+
+	if (action & F_BEGIN) {
+		for (i = 0; i < NR_ACT; i++) {
+			if (!id_seq[i])
+				continue;	/* Activity not in file */
+
+			p = get_activity_position(act, id_seq[i], EXIT_IF_NOT_FOUND);
+			if (!IS_SELECTED(act[p]->options))
+				continue;	/* Activity not selected */
+
+			switch (act[p]->id) {
+
+				case A_CPU:
+				case A_PWR_CPU:
+				case A_NET_SOFT:
+					pcp_def_cpu_metrics(act[p]);
+					break;
+
+				case A_PCSW:
+					pcp_def_pcsw_metrics();
+					break;
+
+				case A_IRQ:
+					pcp_def_irq_metrics(act[p]);
+					break;
+
+				case A_SWAP:
+					pcp_def_swap_metrics();
+					break;
+
+				case A_PAGE:
+					pcp_def_paging_metrics();
+					break;
+
+				case A_IO:
+					pcp_def_io_metrics();
+					break;
+
+				case A_MEMORY:
+					pcp_def_memory_metrics(act[p]);
+					break;
+
+				case A_KTABLES:
+					pcp_def_ktables_metrics();
+					break;
+
+				case A_QUEUE:
+					pcp_def_queue_metrics();
+					break;
+
+				case A_SERIAL:
+					pcp_def_serial_metrics(act[p]);
+					break;
+
+				case A_NET_DEV:
+				case A_NET_EDEV:
+					pcp_def_net_dev_metrics(act[p]);
+					break;
+
+				case A_NET_NFS:
+					pcp_def_net_nfs_metrics();
+					break;
+
+				case A_NET_NFSD:
+					pcp_def_net_nfsd_metrics();
+					break;
+
+				case A_NET_SOCK:
+					pcp_def_net_sock_metrics();
+					break;
+
+				case A_NET_IP:
+					pcp_def_net_ip_metrics();
+					break;
+
+				case A_NET_EIP:
+					pcp_def_net_eip_metrics();
+					break;
+
+				case A_NET_ICMP:
+					pcp_def_net_icmp_metrics();
+					break;
+
+				case A_NET_EICMP:
+					pcp_def_net_eicmp_metrics();
+					break;
+
+				case A_NET_TCP:
+					pcp_def_net_tcp_metrics();
+					break;
+
+				case A_NET_ETCP:
+					pcp_def_net_etcp_metrics();
+					break;
+
+				case A_NET_UDP:
+					pcp_def_net_udp_metrics();
+					break;
+
+				case A_NET_SOCK6:
+					pcp_def_net_sock6_metrics();
+					break;
+
+				case A_NET_IP6:
+					pcp_def_net_ip6_metrics();
+					break;
+
+				case A_NET_EIP6:
+					pcp_def_net_eip6_metrics();
+					break;
+
+				case A_NET_ICMP6:
+					pcp_def_net_icmp6_metrics();
+					break;
+
+				case A_NET_EICMP6:
+					pcp_def_net_eicmp6_metrics();
+					break;
+
+				case A_NET_UDP6:
+					pcp_def_net_udp6_metrics();
+					break;
+			}
+		}
+	}
+#endif /* HAVE_PCP */
+}
+
 /*
  ***************************************************************************
  * Display the "timestamp" part of the report (db and ppc format).
@@ -510,6 +819,7 @@ char *print_dbppc_timestamp(int fmt, struct file_header *file_hdr, char *cur_dat
  * @cur_date	Date string of current record.
  * @cur_time	Time string of current record.
  * @itv		Interval of time with preceding record.
+ * @record_hdr	Record header for current sample (unused here).
  * @file_hdr	System activity file standard header.
  * @flags	Flags for common options.
  *
@@ -519,6 +829,7 @@ char *print_dbppc_timestamp(int fmt, struct file_header *file_hdr, char *cur_dat
  */
 __tm_funct_t print_ppc_timestamp(void *parm, int action, char *cur_date,
 				 char *cur_time, unsigned long long itv,
+				 struct record_header *record_hdr,
 				 struct file_header *file_hdr, unsigned int flags)
 {
 	int utc = !PRINT_LOCAL_TIME(flags) && !PRINT_TRUE_TIME(flags);
@@ -545,6 +856,7 @@ __tm_funct_t print_ppc_timestamp(void *parm, int action, char *cur_date,
  * @cur_date	Date string of current record.
  * @cur_time	Time string of current record.
  * @itv		Interval of time with preceding record.
+ * @record_hdr	Record header for current sample (unused here).
  * @file_hdr	System activity file standard header.
  * @flags	Flags for common options.
  *
@@ -554,6 +866,7 @@ __tm_funct_t print_ppc_timestamp(void *parm, int action, char *cur_date,
  */
 __tm_funct_t print_db_timestamp(void *parm, int action, char *cur_date,
 				char *cur_time, unsigned long long itv,
+				struct record_header *record_hdr,
 				struct file_header *file_hdr, unsigned int flags)
 {
 	int utc = !PRINT_LOCAL_TIME(flags) && !PRINT_TRUE_TIME(flags);
@@ -580,12 +893,14 @@ __tm_funct_t print_db_timestamp(void *parm, int action, char *cur_date,
  * @cur_date	Date string of current comment.
  * @cur_time	Time string of current comment.
  * @itv		Interval of time with preceding record.
+ * @record_hdr	Record header for current sample (unused here).
  * @file_hdr	System activity file standard header (unused here).
  * @flags	Flags for common options.
  ***************************************************************************
  */
 __tm_funct_t print_xml_timestamp(void *parm, int action, char *cur_date,
 				 char *cur_time, unsigned long long itv,
+				 struct record_header *record_hdr,
 				 struct file_header *file_hdr, unsigned int flags)
 {
 	int utc = !PRINT_LOCAL_TIME(flags) && !PRINT_TRUE_TIME(flags);
@@ -612,12 +927,14 @@ __tm_funct_t print_xml_timestamp(void *parm, int action, char *cur_date,
  * @cur_date	Date string of current comment.
  * @cur_time	Time string of current comment.
  * @itv		Interval of time with preceding record.
+ * @record_hdr	Record header for current sample (unused here).
  * @file_hdr	System activity file standard header (unused here).
  * @flags	Flags for common options.
  ***************************************************************************
  */
 __tm_funct_t print_json_timestamp(void *parm, int action, char *cur_date,
 				  char *cur_time, unsigned long long itv,
+				  struct record_header *record_hdr,
 				  struct file_header *file_hdr, unsigned int flags)
 {
 	int utc = !PRINT_LOCAL_TIME(flags) && !PRINT_TRUE_TIME(flags);
@@ -649,6 +966,7 @@ __tm_funct_t print_json_timestamp(void *parm, int action, char *cur_date,
  * @cur_date	Date string of current record.
  * @cur_time	Time string of current record.
  * @itv		Interval of time with preceding record (unused here).
+ * @record_hdr	Record header for current sample (unused here).
  * @file_hdr	System activity file standard header (unused here).
  * @flags	Flags for common options.
  *
@@ -657,8 +975,9 @@ __tm_funct_t print_json_timestamp(void *parm, int action, char *cur_date,
  ***************************************************************************
  */
 __tm_funct_t print_raw_timestamp(void *parm, int action, char *cur_date,
-				char *cur_time, unsigned long long itv,
-				struct file_header *file_hdr, unsigned int flags)
+				 char *cur_time, unsigned long long itv,
+				 struct record_header *record_hdr,
+				 struct file_header *file_hdr, unsigned int flags)
 {
 	int utc = !PRINT_LOCAL_TIME(flags) && !PRINT_TRUE_TIME(flags);
 	static char pre[80];
@@ -672,6 +991,36 @@ __tm_funct_t print_raw_timestamp(void *parm, int action, char *cur_date,
 	return NULL;
 }
 
+/*
+ ***************************************************************************
+ * Display the "timestamp" part of the report (PCP format).
+ *
+ * IN:
+ * @parm	Pointer on specific parameters (unused here).
+ * @action	Action expected from current function.
+ * @cur_date	Date string of current record (unused here).
+ * @cur_time	Time string of current record (unused here).
+ * @itv		Interval of time with preceding record (unused here).
+ * @record_hdr	Record header for current sample.
+ * @file_hdr	System activity file standard header (unused here).
+ * @flags	Flags for common options.
+ *
+ * RETURNS:
+ * Pointer on the "timestamp" string.
+ ***************************************************************************
+ */
+__tm_funct_t print_pcp_timestamp(void *parm, int action, char *cur_date,
+				 char *cur_time, unsigned long long itv,
+				 struct record_header *record_hdr,
+				 struct file_header *file_hdr, unsigned int flags)
+{
+	if (action & F_END) {
+		pcp_write_data(record_hdr, flags);
+	}
+
+	return NULL;
+}
+
 /*
  ***************************************************************************
  * Display the header of the report (XML format).
@@ -679,7 +1028,7 @@ __tm_funct_t print_raw_timestamp(void *parm, int action, char *cur_date,
  * IN:
  * @parm	Specific parameter. Here: number of tabulations.
  * @action	Action expected from current function.
- * @dfile	Name of system activity data file.
+ * @dfile	Unused here (PCP archive file).
  * @file_magic	System activity file magic header.
  * @file_hdr	System activity file standard header.
  * @act		Array of activities (unused here).
@@ -747,7 +1096,7 @@ __printf_funct_t print_xml_header(void *parm, int action, char *dfile,
  * IN:
  * @parm	Specific parameter. Here: number of tabulations.
  * @action	Action expected from current function.
- * @dfile	Name of system activity data file.
+ * @dfile	Unused here (PCP archive file).
  * @file_magic	System activity file magic header.
  * @file_hdr	System activity file standard header.
  * @act		Array of activities (unused here).
@@ -808,7 +1157,7 @@ __printf_funct_t print_json_header(void *parm, int action, char *dfile,
  * IN:
  * @parm	Specific parameter (unused here).
  * @action	Action expected from current function.
- * @dfile	Name of system activity data file.
+ * @dfile	Name of system activity data file (unused here).
  * @file_magic	System activity file magic header.
  * @file_hdr	System activity file standard header.
  * @act		Array of activities.
@@ -1005,6 +1354,72 @@ __printf_funct_t print_svg_header(void *parm, int action, char *dfile,
 	}
 }
 
+/*
+ ***************************************************************************
+ * PCP header function.
+ *
+ * IN:
+ * @parm	Specific parameter (unused here).
+ * @action	Action expected from current function.
+ * @dfile	Name of PCP archive file.
+ * @file_magic	System activity file magic header (unused here).
+ * @file_hdr	System activity file standard header (unused here).
+ * @act		Array of activities (unused here).
+ * @id_seq	Activity sequence (unused here).
+ * @file_actlst	List of (known or unknown) activities in file (unused here).
+ ***************************************************************************
+ */
+__printf_funct_t print_pcp_header(void *parm, int action, char *dfile,
+				  struct file_magic *file_magic,
+				  struct file_header *file_hdr,
+				  struct activity *act[], unsigned int id_seq[],
+				  struct file_activity *file_actlst)
+{
+#ifdef HAVE_PCP
+	char buf[64];
+	struct tm lrectime;
+	unsigned long long utc_sec = file_hdr->sa_ust_time;
+
+	if (action & F_BEGIN) {
+		/* Create new PCP context */
+		pmiStart(dfile, FALSE);
+
+		if (PRINT_LOCAL_TIME(flags)) {
+			tzset();	/* Set timezone value in tzname */
+			pmiSetTimezone(tzname[0]);
+		}
+		else {
+			pmiSetTimezone("UTC");
+		}
+
+		/* Save hostname */
+		pmiSetHostname(file_hdr->sa_nodename);
+
+		/* Save number of CPU in PCP archive */
+		pmiAddMetric("hinv.ncpu",
+			     PM_IN_NULL, PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE,
+			     pmiUnits(0, 0, 1, 0, 0, PM_COUNT_ONE));
+		snprintf(buf, sizeof(buf), "%u",
+			 file_hdr->sa_cpu_nr > 1 ? file_hdr->sa_cpu_nr - 1 : 1);
+		pmiPutValue("hinv.ncpu", NULL, buf);
+	}
+
+	if (action & F_END) {
+		if (action & F_BEGIN) {
+			/* Only the header data will be written to PCP archive */
+			if (!PRINT_LOCAL_TIME(flags)) {
+				/* Convert a time_t value from local time to UTC */
+				if (gmtime_r((const time_t *) &(file_hdr->sa_ust_time), &lrectime)) {
+					utc_sec = mktime(&lrectime);
+				}
+			}
+			pmiWrite(utc_sec, 0);
+		}
+		pmiEnd();
+	}
+#endif
+}
+
 /*
  ***************************************************************************
  * Count the number of new network interfaces in current sample. If a new
