@@ -48,7 +48,7 @@ char *sccsid(void) { return (SCCSID); }
 long interval = -1, count = 0;
 
 /* TRUE if a header line must be printed */
-int dis = TRUE;
+int dish = TRUE;
 /* TRUE if data read from file don't match current machine's endianness */
 int endian_mismatch = FALSE;
 /* TRUE if file's data come from a 64 bit machine */
@@ -548,7 +548,7 @@ void write_stats_startup(int curr)
 	}
 
 	flags |= S_F_SINCE_BOOT;
-	dis = TRUE;
+	dish = TRUE;
 
 	write_stats(curr, USE_SADC, &count, NO_TM_START, NO_TM_END, NO_RESET,
 		    ALL_ACTIVITIES, TRUE);
@@ -602,10 +602,12 @@ int sa_read(void *buffer, size_t size)
  * @cur_time	Time string of current restart message.
  * @utc		True if @cur_time is expressed in UTC (unused here).
  * @file_hdr	System activity file standard header.
+ * @record_hdr	Current record header (unused here).
  ***************************************************************************
  */
 __printf_funct_t print_sar_restart(int *tab, int action, char *cur_date, char *cur_time,
-				  int utc, struct file_header *file_hdr)
+				  int utc, struct file_header *file_hdr,
+				  struct record_header *record_hdr)
 {
 	char restart[64];
 
@@ -628,10 +630,12 @@ __printf_funct_t print_sar_restart(int *tab, int action, char *cur_date, char *c
  * @utc		True if @cur_time is expressed in UTC (unused here).
  * @comment	Comment to display.
  * @file_hdr	System activity file standard header (unused here).
+ * @record_hdr	Current record header (unused here).
  ***************************************************************************
  */
 __print_funct_t print_sar_comment(int *tab, int action, char *cur_date, char *cur_time, int utc,
-				  char *comment, struct file_header *file_hdr)
+				  char *comment, struct file_header *file_hdr,
+				  struct record_header *record_hdr)
 {
 	printf("%-11s", cur_time);
 	cprintf_s(IS_COMMENT, "  COM %s\n", comment);
@@ -786,10 +790,10 @@ void handle_curr_act_stats(int ifd, off_t fpos, int *curr, long *cnt, int *eosaf
 
 		if ((lines >= rows) || !lines) {
 			lines = 0;
-			dis = 1;
+			dish = 1;
 		}
 		else
-			dis = 0;
+			dish = 0;
 
 		if (!*eosaf && (rtype != R_RESTART)) {
 
@@ -987,7 +991,7 @@ void read_stats_from_file(char from_file[])
 	rows = get_win_height();
 
 	/* Read file headers and activity list */
-	check_file_actlst(&ifd, from_file, act, &file_magic, &file_hdr,
+	check_file_actlst(&ifd, from_file, act, flags, &file_magic, &file_hdr,
 			  &file_actlst, id_seq, FALSE, &endian_mismatch, &arch_64);
 
 	/* Perform required allocations */
@@ -1199,8 +1203,8 @@ void read_stats(void)
 
 		/* Print results */
 		if (!dis_hdr) {
-			dis = lines / rows;
-			if (dis) {
+			dish = lines / rows;
+			if (dish) {
 				lines %= rows;
 			}
 			lines++;
@@ -1228,7 +1232,7 @@ void read_stats(void)
 	 * At least one line of stats must have been displayed for this.
 	 * (There may be no lines at all if we press Ctrl/C immediately).
 	 */
-	dis = dis_hdr;
+	dish = dis_hdr;
 	if (avg_count) {
 		write_stats_avg(curr, USE_SADC, ALL_ACTIVITIES);
 	}
